/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.generator.common.utils.SourceModelElementFactory.createStaticContentSourceUnit;

import java.util.List;
import java.util.Map;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.component.model.Port;
import org.fortiss.af3.component.model.PortSpecification;
import org.fortiss.af3.generator.common.model.source.SourcePackage;
import org.fortiss.af3.platform.language.executable.TransmissionUnitExecutableBase;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.TransmissionUnit;
import org.fortiss.af3.platform.raspberry.generated.files.RaspberryPortTypeGeneration;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class BusExecutable extends TransmissionUnitExecutableBase<TransmissionUnit> {

	/**
	 * @param modelElement
	 */
	public BusExecutable(TransmissionUnit modelElement) {
		super(modelElement);
		// TODO Auto-generated constructor stub
	}

	/** {@inheritDoc} */
	@Override
	public void createAllFiles(SourcePackage pack, Map<ExecutionUnit, List<Port>> euPortsPair,
			Map<ExecutionUnit, List<Component>> euComponentList, Component topComponent,
			int deploymentID) {
		checkTypesOfOutputPorts(pack, euPortsPair);
	}

	/** Collects the information about type and bounds of all OutputPorts. */
	public void checkTypesOfOutputPorts(SourcePackage pack,
			Map<ExecutionUnit, List<Port>> portsToCheck) {

		// TODO: Look at all ports, which are OutputPorts (instanceof OutputPort)
		// save for each port the type (i.e.: int) and the upper+lower bounds
		// create a simple string template for the output
		// String Keys;
		String logPortSpecs = new String();
		String portName = new String();
		String portType = new String();

		Object[] ecus = portsToCheck.keySet().toArray();
		for(Object ecu : ecus) { // Iterate over number of Cores
			for(Object port : portsToCheck.get(ecu)) { // Iterate over Number of Ports.
				if(port instanceof InputPort) {
					portName = ((InputPort)port).getName();
					PortSpecification portSpecIn =
							(PortSpecification)((InputPort)port).getSpecifications().get(0);
					portType = portSpecIn.getType().toString();
					// logPortSpecification(portName, portSpecIn, "Input");
					logPortSpecs +=
							"Input PortName:  " + portName + "\nType:      " + portType + "\n\n";

				} else if(port instanceof OutputPort) {
					portName = ((OutputPort)port).getName();
					PortSpecification portSpecOut =
							(PortSpecification)((OutputPort)port).getSpecifications().get(0);
					portType = portSpecOut.getType().toString();
					// logPortSpecification(portName, portSpecOut, "Output");
					logPortSpecs +=
							"Output PortName:  " + portName + "\nType:      " + portType + "\n\n";

				} else {
					System.out.println("Not a port!");
				}
			}
		}
		RaspberryPortTypeGeneration rspPortInfoGen = new RaspberryPortTypeGeneration(logPortSpecs);
		rspPortInfoGen.createInfofile();
		pack.addUnit(createStaticContentSourceUnit("PortSpecifications",
				rspPortInfoGen.getContent(), false));

	}

}
